/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global NX, Ext, Nexus*/

/**
 * Manages activity/event detail mapping.
 *
 * @since 2.3
 *
 * @singleton
 */
NX.define('Nexus.staging.repository.activity.DetailManager', {
    singleton: true,

    mixins: [
        'Nexus.LogAwareMixin'
    ],

    requires: [
        'Nexus.staging.repository.activity.ActivityDetail',
        'Nexus.staging.repository.activity.EventDetail',

        // FIXME: Duplication here w/ *details is painful, how can we not require this?

        // repository*
        'Nexus.staging.repository.activity.EventDetail_repositoryCreated',
        'Nexus.staging.repository.activity.EventDetail_repositoryClosed',
        'Nexus.staging.repository.activity.EventDetail_repositoryCloseFailed',
        'Nexus.staging.repository.activity.EventDetail_repositoryPromoted',
        'Nexus.staging.repository.activity.EventDetail_repositoryPromoteFailed',
        'Nexus.staging.repository.activity.EventDetail_repositoryReleased',
        'Nexus.staging.repository.activity.EventDetail_repositoryReleaseFailed',
        'Nexus.staging.repository.activity.EventDetail_repositoryDropFailed',

        // group*
        'Nexus.staging.repository.activity.EventDetail_repositoryGroupCreated',

        // rules*
        'Nexus.staging.repository.activity.EventDetail_rulesEvaluate',
        'Nexus.staging.repository.activity.EventDetail_rulesPassed',
        'Nexus.staging.repository.activity.EventDetail_rulesFailed',

        // rule*
        'Nexus.staging.repository.activity.EventDetail_ruleEvaluate',
        'Nexus.staging.repository.activity.EventDetail_rulePassed',
        'Nexus.staging.repository.activity.EventDetail_ruleFailed'
    ],

    /**
     * Event name to 'activity' detail class (either class-name or class-instance (after cached).
     *
     * @private
     */
    activityDetails: {
        // for now, all activities will use the basic ActivityDetail
    },

    /**
     * Event name to 'event' detail class (either class-name or class-instance.
     *
     * @private
     */
    eventDetails: {
        // FIXME: Must keep in sync with 'requires' above

        // repository*
        'repositoryCreated':        'Nexus.staging.repository.activity.EventDetail_repositoryCreated',
        'repositoryGroupCreated':   'Nexus.staging.repository.activity.EventDetail_repositoryGroupCreated',
        'repositoryClosed':         'Nexus.staging.repository.activity.EventDetail_repositoryClosed',
        'repositoryCloseFailed':    'Nexus.staging.repository.activity.EventDetail_repositoryCloseFailed',
        'repositoryPromoted':       'Nexus.staging.repository.activity.EventDetail_repositoryPromoted',
        'repositoryPromoteFailed':  'Nexus.staging.repository.activity.EventDetail_repositoryPromoteFailed',
        'repositoryReleased':       'Nexus.staging.repository.activity.EventDetail_repositoryReleased',
        'repositoryReleaseFailed':  'Nexus.staging.repository.activity.EventDetail_repositoryReleaseFailed',
        'repositoryDropFailed':     'Nexus.staging.repository.activity.EventDetail_repositoryDropFailed',

        // rules*
        'rulesEvaluate':        'Nexus.staging.repository.activity.EventDetail_rulesEvaluate',
        'rulesPassed':          'Nexus.staging.repository.activity.EventDetail_rulesPassed',
        'rulesFailed':          'Nexus.staging.repository.activity.EventDetail_rulesFailed',

        // rule*
        'ruleEvaluate':         'Nexus.staging.repository.activity.EventDetail_ruleEvaluate',
        'rulePassed':           'Nexus.staging.repository.activity.EventDetail_rulePassed',
        'ruleFailed':           'Nexus.staging.repository.activity.EventDetail_ruleFailed'
    },

    /**
     * @private
     */
    getDetailType: function (type, name) {
        var self = this,
            details,
            detailType;

        self.logDebug('Looking up ' + type + ' detail: ' + name);

        switch (type) {
            case 'activity':
                details = self.activityDetails;
                break;
            case 'event':
                details = self.eventDetails;
                break;
            default:
                throw new Error('Invalid type: ' + type);
        }

        detailType = details[name];

        // select a default detail type
        if (detailType === undefined) {
            self.logDebug('No detail class configured for ' + type + ': ' + name + '; using default');

            switch (type) {
                case 'activity':
                    detailType = 'Nexus.staging.repository.activity.ActivityDetail';
                    break;
                case 'event':
                    detailType = 'Nexus.staging.repository.activity.EventDetail';
                    break;
                // default already covered
            }
        }

        // Reference detail type class and cache it
        if (Ext.isString(detailType)) {
            var className = detailType;
            detailType = NX.obj(className);
            details[name] = detailType;
            self.logDebug('Cached ' + type + ' detail type: ' + className);
        }

        return detailType;
    },

    /**
     * Return activity detail class.
     */
    forActivity: function (activity) {
        var detailType = this.getDetailType('activity', activity.name);
        return new detailType(activity);
    },

    /**
     * Return event detail class.
     */
    forEvent: function (event) {
        var detailType = this.getDetailType('event', event.name);
        return new detailType(event);
    }
});
