/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/**
 * Dialog to authenticate current user.
 *
 * @class Nexus.usertoken.AuthenticateWindow
 * @extends Ext.Window
 * @namespace Nexus.usertoken
 */
Ext.define('Nexus.usertoken.AuthenticateWindow', {
    extend: 'Ext.Window',
    requires: ['Nexus.Image'],
    requirejs: ['Nexus/usertoken'],
    mixins: [
      'Nexus.LogAwareMixin'
    ],

    constructor: function (config) {
        var self = this;

        Ext.applyIf(config, {
            message: 'You have requested an operation which requires validation of your credentials.'
        });

        self.callbackRef = new Nexus.CallbackRef(config.callback);

        self.passwordField = new Ext.form.TextField({
            fieldLabel: 'Password',
            inputType: 'password',
            name: 'password',
            width: 170,
            allowBlank: false,
            validateOnBlur: false // allow cancel to be clicked w/o validating this to be non-blank
        });

        Ext.apply(self, {
            cls: 'nx-usertoken-AuthenticateWindow',
            title: 'Authenticate',
            border: false,
            closable: false,
            modal: true,
            autoHeight: true,
            width: 350,
            resizable: false,
            items: [
                {
                    xtype: 'form',
                    autoHeight: true,
                    labelAlign: 'right',
                    frame: true,
                    monitorValid: true,
                    layoutConfig: {
                        labelSeparator: ''
                    },
                    items: [
                        {
                            xtype: 'image',
                            cls: 'nx-usertoken-icon',
                            src: Nexus.usertoken.iconPath('lock-32.png')
                        },
                        {
                            xtype: 'label',
                            html: config.message + '<br style="clear:both;"/>' // clear to get remaining form elements aligned correctly
                        },
                        {
                            xtype: 'textfield',
                            fieldLabel: 'Username',
                            name: 'username',
                            width: 170,
                            readOnly: true
                        },
                        self.passwordField
                    ],
                    buttons: [
                        {
                            text: 'Confirm',
                            formBind: true,
                            scope: self,
                            handler: self.doValidate
                        },
                        new Nexus.ext.LinkButton({
                            text: 'Cancel',
                            formBind: false,
                            scope: self,
                            handler: self.close
                        })
                    ],
                    keys: [
                        {
                            // Validate on ENTER
                            key: Ext.EventObject.ENTER,
                            scope: self,
                            fn: self.doValidate
                        },
                        {
                            // Close dialog on ESC
                            key: Ext.EventObject.ESC,
                            scope: self,
                            fn: self.close
                        }
                    ]
                }
            ],
            listeners: {
                show: function (component) {
                    component.find('name', 'username')[0].setValue(Nexus.currentUser().username);
                    component.find('name', 'password')[0].focus(false, 100);
                }
            }
        });

        // Call super constructor
        self.constructor.superclass.constructor.apply(self, arguments);
    },

    /**
     * Validate provided authentication credentials and invoke callback.
     *
     * @private
     */
    doValidate: function () {
        var self = this, mask;

        mask = new Ext.LoadMask(self.getEl(), {
            msg: 'Authenticating'
        });
        mask.show();

        Ext.Ajax.request({
            url: Nexus.usertoken.resourcePath('/authenticate'),
            method: 'POST',
            suppressStatus: [
                403 // Used to signal from the server unauthenticated state, don't display an error box for this response
            ],

            jsonData: {
                u: Sonatype.utils.base64.encode(Nexus.currentUser().username),
                p: Sonatype.utils.base64.encode(self.passwordField.getValue())
            },

            callback: function () {
                mask.hide();
            },

            success: function (response) {
                self.logDebug('Authenticated');

                var authTicket = Ext.decode(response.responseText);
                self.logDebug('Ticket: ' + authTicket.t);

                // Close the dialog
                self.close();

                // invoke callback with the authentication ticket
                self.callbackRef.invoke(authTicket.t);
            },

            failure: function (response) {
                if (response.status === 403) {
                    self.passwordField.markInvalid('Invalid password');
                }
            }
        });
    }
});
