/*
 * Copyright (c) 2008-2013 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global $,console*/

/**
 * Add global Nexus variable if it does not exist yet.
 */
var Nexus;
if (!Nexus) {
  Nexus = {};
}

(function() {
  "use strict";

  var handlers = {}, target,
        receiveMessage, localError;

  receiveMessage = function(event) {
    var json, callbacks, len, i;

    if (event.data) {
      try {
        json = JSON.parse(event.data);
      }
      catch (e) {
        if (console) {
          console.log('Could not parse event payload:\n' + e.description + '\n' + e);
        }
        Nexus.error('payload', e);
        return;
      }

      callbacks = handlers[json.id];
      if ((callbacks === undefined || (len = callbacks.length) === 0) && json.id !== 'unknown-message') {
        Nexus.send('unknown-message', json.id);
      }

      for (i = 0; i < len; i = i += 1) {
        if (callbacks.hasOwnProperty(i)) {
          callbacks[i].call(callbacks[i], json.id, json.data);
        }
      }
    }
  };

  localError = function(id, msg) {
    receiveMessage({ data : { id : 'error', payload : { id : id, msg : msg }}});
  };

  /**
   * @param targetWindow the window to send messages to.
   * @static
   */
  Nexus.init = function(targetWindow) {
    target = targetWindow;
    handlers = {
      error : [
            function(id, payload) {
              if (console && console.log) {
                console.log('Received error: ' + payload);
              }
            }
      ]
    };
  };

  /**
   * Register handler for an event.
   * @param {String} evtName the event to listen to
   * @param {Function} handler the callback to execute: function({String}evtId, {Object}payload)
   * @static
   */
  Nexus.on = function(evtName, handler) {
    if (!handlers[evtName]) {
      handlers[evtName] = [];
    }

    handlers[evtName].push(handler);
  };

  /**
   * Unregister handler for an event.
   * @param {String} evtName the event to listen to
   * @param {Function} handler the callback to remove
   * @static
   */
  Nexus.un = function(evtName, handler) {
    var idx, rest, evtHandlers = handlers[evtName];

    if (!evtHandlers) {
      handlers[evtName] = [];
      evtHandlers = handlers[evtName];
    }

    idx = evtHandlers.indexOf(handler);
    rest = evtHandlers.slice(idx + 1 || evtHandlers.length);
    evtHandlers.length = idx;
    evtHandlers.push.apply(evtHandlers, rest);
  };

  /**
   * Sends an event to the target window.
   *
   * @param {String} evtId The event id.
   * @param {String/Object} data The data to send with the event.
   * @see Nexus.init
   * @static
   */
  Nexus.send = function(evtId, data) {
    if (target === undefined) {
      throw new Error("No target defined to send messages to. Call Nexus.init first.");
    }

    if (target.postMessage === undefined) {
      localError('fatal', 'Target cannot receive messages.');
      return;
    }

    var payload = {
      id : evtId
    };

    if (data !== undefined) {
      payload.data = data;
    }

    target.postMessage(JSON.stringify(payload), '*');
  };

  /**
   * Send an error message to the target window.
   * @param {String} errorId The error id.
   * @param {String/Object} data The data to send as error message.
   */
  Nexus.error = function(errorId, data) {
    Nexus.send('error', {
      id : errorId,
      msg : data
    });
  };

  if (window.attachEvent) {
    // Internet Explorer
    window.attachEvent('onmessage', receiveMessage);
  } else if (window.addEventListener) {
    window.addEventListener("message", receiveMessage, false);
  } else {
    localError('fatal', 'Could not install message listener.');
  }

}());

